@@ -10,43 +10,61 @@
 #' @return The rounded vector or matrix.
 #' @seealso \link{round}
 #' @examples
-#' # a vector that is off by 1e-8
-#' x <- c(0.25 + 1e-8, 0.25, 0.5)
+#' # regular rounding would not sum up to 1 
+#' x <- c(0.333, 0.334, 0.333)
 #' 
 #' round_stochastic(x)
 #' round_stochastic(x, digits = 2)
 #' round_stochastic(x, digits = 1)
 #' round_stochastic(x, digits = 0)
+#' 
+#' 
+#' # round a stochastic matrix
+#' m <- matrix(runif(15), ncol = 3)
+#' m <- sweep(m, 1, rowSums(m), "/")
+#' 
+#' m
+#' round_stochastic(m, digits = 2)
+#' round_stochastic(m, digits = 1)
+#' round_stochastic(m, digits = 0)
 #' @export
-round_stochastic <- function(x, digits = getOption("digits")) {
+round_stochastic <- function(x, digits = 7) {
   if (is.matrix(x))
     t(apply(
       x,
       MARGIN = 1,
-      .round_preserve_sum,
+      round_stochastic_int,
       digits = digits
     ))
   else
-    .round_preserve_sum(x, digits = digits)
+    round_stochastic_int(x, digits = digits)
 }
 
 # Round a stochastic vector
-# from https://github.com/larmarange/JLutils/blob/master/R/round_preserve_sum.R
-.round_preserve_sum <- function(x, digits = getOption("digits")) {
-  up <- 10 ^ digits
-  x <- x * up
-  y <- floor(x)
-  indices <- tail(order(x - y), round(sum(x)) - sum(y))
-  y[indices] <- y[indices] + 1
-  r <- y / up
-  if (!sum1(r, digits))
-    warning("The rounded vector is not stochastic.")
-  r
+round_stochastic_int <- function(x, digits = 7) {
+  # handle impossible beliefs
+  if (any(is.na(x)))
+    return (x)
+  
+  x[x<0] <- 0
+  
+  #x / sum(x)
+  xr <- round(x, digits = digits)
+  s <- sum(xr)
+  if (s != 1) {
+    inc_id <- which.max(x - xr)
+    xr[inc_id] <- xr[inc_id] + 1 - s
+  }
+  
+  xr
 }
 
+
+# check if vector sums to 1
 sum1 <- function(x, digits = getOption("digits")) {
   if(is.matrix(x))
     all(apply(x, MARGIN = 1, sum1))
   else
     zapsmall(sum(x), digits) == 1
 }
+
